/*----------------------------------------------------------------------------

File: _getfile.p

Description:

    Roundtable intercept of Common GetFile Dialog.

    When Roundtable is active, this routine will be in the PROPATH before
    Progress's standard _getfile.p.

    We first publish Roundtable's "evRtbAdeGetFile" event which will tell
    this program to
      1) use the object selected within Roundtable, or
      2) cancel the get file operation, or
      3) carry on with this program, to get a file not managed by Roundtable.

    Roundtable only intercepts if the product is UIB or Procedure.

Input Parameters:
    p_Window  : Window in which to display the dialog box.
    p_product : The ADE product code.  eg. "uib" for the UIB, or "Procedure"
                for the Procedure Window.
    p_action  : A string indicating what the intent of this call is.
                eg.  "Open", "Save As", "Export As"
                [This is often identical to the p_title for the dialog].
    p_title   : The title to use for the system-dialog.
    p_mode    : "OPEN" or "SAVE".  This determines which version of
                SYSTEM-DIALOG to use:
                    p_mode  Options
                    ------  -------
                    OPEN -> MUST-EXIT
                    SAVE -> SAVE-AS USE-FILENAME ASK-OVERWRITE CREATE-TEST-FILE

Input-Output Parameters:
    p_filename: The filename to use as the default, and to return.

Output Parameters:
    p_ok      : TRUE if user successfully choose a file name (the result of the
                SYSTEM-DIALOG GET-FILE...UPDATE p_ok.

Side Effects:
    <none>

----------------------------------------------------------------------------*/
DEFINE INPUT        PARAMETER p_Window   AS WIDGET-HANDLE NO-UNDO.
DEFINE INPUT        PARAMETER p_product  AS CHAR    NO-UNDO.
DEFINE INPUT        PARAMETER p_action   AS CHAR    NO-UNDO.
DEFINE INPUT        PARAMETER p_title    AS CHAR    NO-UNDO.
DEFINE INPUT        PARAMETER p_mode     AS CHAR    NO-UNDO.
DEFINE INPUT-OUTPUT PARAMETER p_filename AS CHAR    NO-UNDO.
DEFINE       OUTPUT PARAMETER p_ok       AS LOGICAL NO-UNDO.

DEFINE VARIABLE Mgetfile    AS CHARACTER NO-UNDO.
DEFINE VARIABLE Mrtb-action AS CHARACTER NO-UNDO.
DEFINE VARIABLE MhTabletop  AS HANDLE    NO-UNDO.

/*
message
  "p_Window" p_Window
  skip
  "p_product" p_product
  skip
  "p_action" p_action
  skip
  "p_title" p_title
  skip
  "p_mode" p_mode
  skip
  "p_filename" p_filename VIEW-AS ALERT-BOX.
*/
PUBLISH "evRtbGetDeskHandle":U (OUTPUT MhTabletop).

IF VALID-HANDLE(MhTabletop) AND
    (CAN-DO("UIB,Procedure",p_product) OR p_product = ?) THEN
DO:

    PUBLISH "evRtbAdeGetFile":U (INPUT  p_Window ,
        INPUT  p_product,
        INPUT  p_action,
        INPUT  p_title,
        INPUT  p_mode,
        INPUT-OUTPUT p_filename,
        OUTPUT Mrtb-action).

    CASE Mrtb-action:
        WHEN "Cancel" OR
        WHEN "" THEN
            DO:
                ASSIGN
                    p_ok = NO.
                RETURN.
            END.
        WHEN "Go" THEN
            DO:
                ASSIGN
                    p_ok = YES.
                RETURN.
            END.
        OTHERWISE
        DO: /* nothing */
        END.
    END CASE.
END.   /* can-do... */

/*
 If we got this far, then we want to run PSC's getfile dialog.

 Use the path to _adeload.r as the full path to _getfile.r.
 (It is supposed to be in the same adecomm procedure library, and Roundtable
 has no intercept for _adeload.r.)

 Don't use GET-KEY-VALUE to get DLC, just in case something weird has been
 done with the path to GUI/ADECOMM.
*/
ASSIGN
    Mgetfile = SEARCH("adecomm/_adeload.r").
IF Mgetfile = ? THEN
DO:
    MESSAGE "Roundtable was unable to find adecomm/_adeload.r." SKIP
        "Please ensure that your propath to PROGRESS is set correctly."
        VIEW-AS ALERT-BOX ERROR.
    RETURN.
END.
ASSIGN
    Mgetfile = REPLACE(Mgetfile,"_adeload","_getfile").
IF SEARCH(Mgetfile) = ? THEN
DO:
    MESSAGE "Roundtable was unable to find adecomm/_getfile.r." SKIP
        "Please ensure that your propath to PROGRESS is set correctly."
        VIEW-AS ALERT-BOX ERROR.
    RETURN.
END.

/*
 Note: p_filename may have been set to contain UNIX style slashes ("/")
 by Roundtable, but that will break the SYSTEM-DIALOG GETFILE. So, when
 we run GUI/ADECOMM/_getfile.r, we make sure that we pass it p_filename
 with "\" instead of "/".
*/
ASSIGN
    p_filename = REPLACE(p_filename,"/","~\").
RUN VALUE( Mgetfile )
    ( INPUT        p_Window,
    INPUT        p_product,
    INPUT        p_action,
    INPUT        p_title,
    INPUT        p_mode,
    INPUT-OUTPUT p_filename,
    OUTPUT       p_ok ).

/*
 Reset the Action in rtb_adelib.p if user chooses cancel..
*/
IF p_ok = NO THEN
DO:
    DYNAMIC-FUNCTION('fnRtbSetAdeAction':U,"").
END.
